package geniusweb.events;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.annotation.JsonTypeInfo.As;

/**
 * reports some event happened in the negotiation system. Generally, parties are
 * informed about events in the system, but this depends on the protocol.
 *
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = As.WRAPPER_OBJECT)
@JsonSubTypes({ @JsonSubTypes.Type(value = ActionEvent.class),
		@JsonSubTypes.Type(value = SessionStarted.class),
		@JsonSubTypes.Type(value = TournamentStarted.class) })
public interface NegotiationEvent {
	/**
	 * @return the time at which the event happened on the server, measured in
	 *         milliseconds, between the start time and midnight, January 1,
	 *         1970 UTCas. See also {@link System#currentTimeMillis()}. <br>
	 * 
	 *         If the event is about an {@link Action} done by a negotiation
	 *         party on another machine, this time refers to the time the action
	 *         was handled on the server (which may differ from the clock time
	 *         on the other machine). <br>
	 * 
	 *         Note: we do not use RFC 3339 because we need millisecond
	 *         accuracy.
	 */
	Long getTime();

}
